@@ -26,13 +26,16 @@
 #include <assert.h>
 #include <linux/watchdog.h>
 
+#include <libsmartcols.h>
+
 #include "nls.h"
 #include "c.h"
+#include "xalloc.h"
 #include "closestream.h"
 #include "optutils.h"
 #include "pathnames.h"
 #include "strutils.h"
-#include "tt.h"
+#include "carefulputc.h"
 
 /*
  * since 2.6.18
@@ -56,6 +59,10 @@
 					   other external alarm not a reboot */
 #endif
 
+/* basic output flags */
+static int no_headings;
+static int raw;
+
 struct wdflag {
 	uint32_t	flag;
 	const char	*name;
@@ -82,7 +89,7 @@ static const struct wdflag wdflags[] = {
 struct colinfo {
 	const char *name; /* header */
 	double	   whint; /* width hint (N < 1 is in percent of termwidth) */
-	int	   flags; /* TT_FL_* */
+	int	   flags; /* SCOLS_FL_* */
 	const char *help;
 };
 
@@ -91,15 +98,15 @@ enum { COL_FLAG, COL_DESC, COL_STATUS, COL_BSTATUS, COL_DEVICE };
 /* columns descriptions */
 static struct colinfo infos[] = {
 	[COL_FLAG]    = { "FLAG",        14,  0, N_("flag name") },
-	[COL_DESC]    = { "DESCRIPTION", 0.1, TT_FL_TRUNC, N_("flag description") },
-	[COL_STATUS]  = { "STATUS",      1,   TT_FL_RIGHT, N_("flag status") },
-	[COL_BSTATUS] = { "BOOT-STATUS", 1,   TT_FL_RIGHT, N_("flag boot status") },
+	[COL_DESC]    = { "DESCRIPTION", 0.1, SCOLS_FL_TRUNC, N_("flag description") },
+	[COL_STATUS]  = { "STATUS",      1,   SCOLS_FL_RIGHT, N_("flag status") },
+	[COL_BSTATUS] = { "BOOT-STATUS", 1,   SCOLS_FL_RIGHT, N_("flag boot status") },
 	[COL_DEVICE]  = { "DEVICE",      0.1, 0, N_("watchdog device name") }
 
 };
 
-#define NCOLS ARRAY_SIZE(infos)
-static int columns[NCOLS], ncolumns;
+static int columns[ARRAY_SIZE(infos) * 2];
+static int ncolumns;
 
 struct wdinfo {
 	char		*device;
@@ -136,7 +143,7 @@ static int column2id(const char *name, size_t namesz)
 {
 	size_t i;
 
-	for (i = 0; i < NCOLS; i++) {
+	for (i = 0; i < ARRAY_SIZE(infos); i++) {
 		const char *cn = infos[i].name;
 		if (!strncasecmp(name, cn, namesz) && !*(cn + namesz))
 			return i;
@@ -147,9 +154,8 @@ static int column2id(const char *name, size_t namesz)
 
 static int get_column_id(int num)
 {
-	assert(ARRAY_SIZE(columns) == NCOLS);
 	assert(num < ncolumns);
-	assert(columns[num] < (int) NCOLS);
+	assert(columns[num] < (int) ARRAY_SIZE(infos));
 
 	return columns[num];
 }
@@ -197,14 +203,14 @@ static void usage(FILE *out)
 	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
-static void add_flag_line(struct tt *tt, struct wdinfo *wd, const struct wdflag *fl)
+static void add_flag_line(struct libscols_table *table, struct wdinfo *wd, const struct wdflag *fl)
 {
 	int i;
-	struct tt_line *line;
+	struct libscols_line *line;
 
-	line = tt_add_line(tt, NULL);
+	line = scols_table_new_line(table, NULL);
 	if (!line) {
-		warn(_("failed to add line to output"));
+		warn(_("failed to initialize output line"));
 		return;
 	}
 
@@ -232,29 +238,33 @@ static void add_flag_line(struct tt *tt, struct wdinfo *wd, const struct wdflag
 		}
 
 		if (str)
-			tt_line_set_data(line, i, str);
+			scols_line_set_data(line, i, str);
 	}
 }
 
-static int show_flags(struct wdinfo *wd, int tt_flags, uint32_t wanted)
+static int show_flags(struct wdinfo *wd, uint32_t wanted)
 {
 	size_t i;
 	int rc = -1;
-	struct tt *tt;
+	struct libscols_table *table;
 	uint32_t flags;
 
+	scols_init_debug(0);
+
 	/* create output table */
-	tt = tt_new_table(tt_flags);
-	if (!tt) {
+	table = scols_new_table();
+	if (!table) {
 		warn(_("failed to initialize output table"));
 		return -1;
 	}
+	scols_table_enable_raw(table, raw);
+	scols_table_enable_noheadings(table, no_headings);
 
 	/* define columns */
 	for (i = 0; i < (size_t) ncolumns; i++) {
 		struct colinfo *col = get_column_info(i);
 
-		if (!tt_define_column(tt, col->name, col->whint, col->flags)) {
+		if (!scols_table_new_column(table, col->name, col->whint, col->flags)) {
 			warnx(_("failed to initialize output column"));
 			goto done;
 		}
@@ -268,7 +278,7 @@ static int show_flags(struct wdinfo *wd, int tt_flags, uint32_t wanted)
 		if (wanted && !(wanted & wdflags[i].flag))
 			; /* ignore */
 		else if (flags & wdflags[i].flag)
-			add_flag_line(tt, wd, &wdflags[i]);
+			add_flag_line(table, wd, &wdflags[i]);
 
 		flags &= ~wdflags[i].flag;
 	}
@@ -276,10 +286,10 @@ static int show_flags(struct wdinfo *wd, int tt_flags, uint32_t wanted)
 	if (flags)
 		warnx(_("%s: unknown flags 0x%x\n"), wd->device, flags);
 
-	tt_print_table(tt);
+	scols_print_table(table);
 	rc = 0;
 done:
-	tt_free_table(tt);
+	scols_unref_table(table);
 	return rc;
 }
 /*
@@ -330,9 +340,11 @@ static int set_watchdog(struct wdinfo *wd, int timeout)
 		warn(_("cannot set timeout for %s"), wd->device);
 	}
 
-	close(fd);
+	if (close_fd(fd))
+		warn(_("write failed"));
 	sigprocmask(SIG_SETMASK, &oldsigs, NULL);
-	printf("Set timeout to %d seconds\n", timeout);
+	printf(P_("Timeout has been set to %d second.\n",
+		  "Timeout has been set to %d seconds.\n", timeout), timeout);
 
 	return rc;
 }
@@ -393,7 +405,8 @@ static int read_watchdog(struct wdinfo *wd)
 		 * the machine might end up rebooting. */
 	}
 
-	close(fd);
+	if (close_fd(fd))
+		warn(_("write failed"));
 	sigprocmask(SIG_SETMASK, &oldsigs, NULL);
 
 	return 0;
@@ -408,7 +421,7 @@ static void print_oneline(struct wdinfo *wd, uint32_t wanted,
 		printf(" VERSION=\"%x\"", wd->ident.firmware_version);
 
 		printf(" IDENTITY=");
-		tt_fputs_quoted((char *) wd->ident.identity, stdout);
+		fputs_quoted((char *) wd->ident.identity, stdout);
 	}
 	if (!notimeouts) {
 		if (wd->has_timeout)
@@ -446,17 +459,20 @@ static void print_oneline(struct wdinfo *wd, uint32_t wanted,
 static void show_timeouts(struct wdinfo *wd)
 {
 	if (wd->has_timeout)
-		printf(_("%-15s%2i seconds\n"), _("Timeout:"), wd->timeout);
+		printf(P_("%-14s %2i second\n", "%-14s %2i seconds\n", wd->timeout),
+			  _("Timeout:"), wd->timeout);
 	if (wd->has_pretimeout)
-		printf(_("%-15s%2i seconds\n"), _("Pre-timeout:"), wd->pretimeout);
+		printf(P_("%-14s %2i second\n", "%-14s %2i seconds\n", wd->pretimeout),
+			  _("Pre-timeout:"), wd->pretimeout);
 	if (wd->has_timeleft)
-		printf(_("%-15s%2i seconds\n"), _("Timeleft:"), wd->timeleft);
+		printf(P_("%-14s %2i second\n", "%-14s %2i seconds\n", wd->timeleft),
+			  _("Timeleft:"), wd->timeleft);
 }
 
 int main(int argc, char *argv[])
 {
 	struct wdinfo wd;
-	int c, tt_flags = 0, res = EXIT_SUCCESS, count = 0;
+	int c, res = EXIT_SUCCESS, count = 0;
 	char noflags = 0, noident = 0, notimeouts = 0, oneline = 0;
 	uint32_t wanted = 0;
 	int timeout = 0;
@@ -523,10 +539,10 @@ int main(int argc, char *argv[])
 			notimeouts = 1;
 			break;
 		case 'n':
-			tt_flags |= TT_FL_NOHEADINGS;
+			no_headings = 1;
 			break;
 		case 'r':
-			tt_flags |= TT_FL_RAW;
+			raw = 1;
 			break;
 		case 'O':
 			oneline = 1;
@@ -594,7 +610,7 @@ int main(int argc, char *argv[])
 		if (!notimeouts)
 			show_timeouts(&wd);
 		if (!noflags)
-			show_flags(&wd, tt_flags, wanted);
+			show_flags(&wd, wanted);
 	} while (optind < argc);
 
 	return res;
